@@ -2,7 +2,7 @@ import numpy as np
 def to_json(state):
     if isinstance(state, np.ndarray):
         return state.tolist()
-    elif isinstance(state, np.int64):
+    elif isinstance(state, np.int64) or isinstance(state, np.int32):
         return state.tolist()
     elif isinstance(state, list):
         return [to_json(s) for s in state]
